var Sequencing = function()
{
	var sequencing = {};
	var debug = 0;
	

	// Page initialization
	sequencing.initPage = function()
	{
		// Step 1
		Events.addEventHandler('seqRuns', 'change', sequencing.sequencingRunOnChange);
		Buttons.addClickHandler('btnCheck', sequencing.checkDataFilesOnClick);
		Events.addEventHandler('outcomeSuccess', 'click', sequencing.outcomeOnChange);
		Events.addEventHandler('outcomeFailed', 'click', sequencing.outcomeOnChange);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', sequencing.initializeStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', sequencing.submit);

		Wizard.showLoadingAnimation('Loading sequencing runs...');	
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnconfirmedSequencingRuns&pipeline='+encodeURIComponent(Data.get('page-data', 'pipeline'));
		Wizard.asyncJsonRequest(url, sequencing.initializeStep1);
	}

	sequencing.initializeStep1 = function(response)
	{
		var seqRuns = response.sequencingRuns;
		var frm = document.forms['reggie'];	

		if (seqRuns.length > 0)
		{
			for (var runNo=0; runNo < seqRuns.length; runNo++)
			{
				var seqRun = seqRuns[runNo];
				var option = sequencing.getOptionForSeqRun(seqRun);
				frm.seqRuns.options[frm.seqRuns.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No sequencing runs available for processing.');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		sequencing.sequencingRunOnChange();
		frm.seqRuns.focus();
	}

	sequencing.sequencingRunOnChange = function()
	{
		var frm = document.forms['reggie'];	
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		
		Doc.element('startDate').innerHTML =  Reggie.reformatDate(seqRun.SequencingStart);
		Doc.element('endDate').innerHTML = Reggie.reformatDate(seqRun.SequencingEnd);
		
		Forms.checkRadio(frm.outcome, seqRun.SequencingResult || 'Successful');
		sequencing.outcomeOnChange();
			
		var fc = seqRun.flowCell;
		Doc.element('flowCellInRun').innerHTML = '<b>['+seqRun.HiSeqPosition+']</b> ' + Strings.encodeTags(fc.FlowCellID);
		Doc.element('flowCellType').innerHTML = fc.FlowCellType || '';
		
		var pools = fc.pools;
		var poolNames = [];
		var libPlateNames = [];
		for (var poolNo = 0; poolNo < pools.length; poolNo++)
		{
			var pool = pools[poolNo];
			poolNames[poolNames.length] = pool.name;
			if (pool.libPlates)
			{
				for (var plateNo = 0; plateNo < pool.libPlates.length; plateNo++)
				{
					var plateName = pool.libPlates[plateNo].name;
					if (libPlateNames.indexOf(plateName) == -1)
					{
						libPlateNames[libPlateNames.length] = plateName;
					}
				}
			}
		}
		Doc.element('pools').innerHTML = Strings.encodeTags(poolNames.join(', '));
		Doc.element('libPlates').innerHTML = Strings.encodeTags(libPlateNames.join(', ')) || 'n/a';
		Doc.element('comments').innerHTML = Strings.encodeTags(seqRun.comments);
	}


	sequencing.outcomeOnChange = function()
	{
		var frm = document.forms['reggie'];	
		frm.flagPools.disabled = !Doc.element('outcomeFailed').checked;
	}
	
	sequencing.getOptionForSeqRun = function(seqRun)
	{
		var flowCell = seqRun.flowCell;
		var name = seqRun.name+': ';
		var numPools = flowCell.pools.length;
		var firstPoolNum = sequencing.getPoolNum(flowCell.pools[0].name);
		var lastPoolNum = sequencing.getPoolNum(flowCell.pools[flowCell.pools.length-1].name);

		if (lastPoolNum - firstPoolNum == numPools - 1)
		{
			if (numPools > 1)
			{
				// Display: PoolN -- PoolY
				name += flowCell.pools[0].name + ' — ' + flowCell.pools[flowCell.pools.length-1].name;
			}
			else
			{
				name += flowCell.pools[0].name;
			}
		}
		else
		{
			// Display: PoolN + x more...
			name += flowCell.pools[0].name + ' + ' + (numPools-1) + ' more...';
		}
		
		var opt = [];
		if (flowCell.FlowCellType) opt[opt.length] = flowCell.FlowCellType;
		opt[opt.length] = seqRun.pipeline;
		name += ' [' + opt.join('; ') + ']';

		// Tooltip is always all pools
		var title = '';
		for (var poolNo=0; poolNo < numPools; poolNo++)
		{
			var pool = flowCell.pools[poolNo];
			if (poolNo > 0) title += ', ';
			title += pool.name;
		}

		var option = new Option(name, seqRun.id);
		option.title = title;
		option.seqRun = seqRun;	
		return option;
	}

	sequencing.getPoolNum = function(poolName)
	{
		var num = poolName.match(/Pool(\d+)/);
		return num ? parseInt(num[1], 10) : null;
	}

	sequencing.checkDataFilesOnClick = function()
	{
		var frm = document.forms['reggie'];
		var url = '../analysis/check_data_files.jsp?ID='+App.getSessionId();
		url += '&seqrun='+frm.seqRuns.value;
		Dialogs.openPopup(url, 'CheckDataFiles', 900, 600);
	}
	
	sequencing.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		var outcome = Forms.getCheckedRadio(frm, 'outcome').value;
		
		frm.sequencingComments.value = seqRun.comments;
		Doc.element('step-2-title').innerHTML = Strings.encodeTags(frm.seqRuns[frm.seqRuns.selectedIndex].text);

		var fc = seqRun.flowCell;
		if (outcome != 'Successful')
		{
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of failure.');
			Doc.hide('failedLanesRow');
		}
		else
		{
			// Failed lanes
			var html = '';
			for (var laneNo = 1; laneNo <= fc.numLanes; laneNo++)
			{
				html += '<label><input type="checkbox" name="failedLanes" value="'+laneNo+'">'+laneNo+'</label> ';
			}
			Doc.element('failedLanes').innerHTML = html;
			Doc.show('failedLanesRow');
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('goregister');
		Doc.show('gocancel');
	}
	
	sequencing.submit = function()
	{

		var frm = document.forms['reggie'];
		
		var seqRun = frm.seqRuns[frm.seqRuns.selectedIndex].seqRun;
		var outcome = Forms.getCheckedRadio(frm, 'outcome').value;

		var submitInfo = {};
		submitInfo.flagPools = frm.flagPools.checked ? true : false;

		seqRun.comments = frm.sequencingComments.value;
		seqRun.SequencingResult = outcome;
		submitInfo.sequencingRun = seqRun;
				
		if (outcome == 'Successful')
		{
			var fc = seqRun.flowCell;
			var failedLanes = [];
			var checkboxes = frm['failedLanes'];
			for (var i = 0; i < checkboxes.length; i++)
			{
				if (checkboxes[i].checked) failedLanes[failedLanes.length] = parseInt(checkboxes[i].value);
			}
			fc.failedLanes = failedLanes;
		}
			
		var url = '../SequencingRun.servlet?ID='+App.getSessionId();
		url += '&cmd=ConfirmSequencingEnded&pipeline='+encodeURIComponent(Data.get('page-data', 'pipeline'));
			
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, sequencing.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	sequencing.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return sequencing;
}();

Doc.onLoad(Sequencing.initPage);

